/*
Example use of RS232 port as stdio and use of scanf and printf
For the Orangutan with the ATmega168 processor. 
Produces about 4500 bytes of loaded code.
*/

#include <stdio.h>
#include <avr/io.h>

// set CPU clock speed and desired baud rate

#define F_CPU 8000000UL
#define BAUDRATE 9600
 
#include <util/delay.h>

// set up uart as io stream
FILE* uart;
int uart_putchar(char c, FILE* stream);
int uart_getchar(FILE* stream);

//initialize the UART hardware on an ATmega168. 

void init_uart(void)
{
   UBRR0H = ((F_CPU / (16UL * BAUDRATE) - 1)>>8);  //0 for 8 MHz
   UBRR0L = ((F_CPU / (16UL * BAUDRATE) - 1)&0xFF);  //51 for 8 MHz
   UCSR0B = (1<<RXEN0) | (1<<TXEN0);  //enable transmitter and receiver
   UCSR0C = (3<<UCSZ00); // 8 bit, 1 stop bit, no parity
}

// put a character to the UART

int uart_putchar(char c,FILE* stream) 
{
   loop_until_bit_is_set(UCSR0A,UDRE0);
   UDR0 = c;
   if(c == '\n') uart_putchar('\r',stream);  // newline -> CRLF
   return 0;
}

//get a character

int uart_getchar(FILE* stream) 
{
   char c;
   loop_until_bit_is_set(UCSR0A,RXC0);
   c = UDR0;
   uart_putchar (c,stream);   //echo it
   return c;
}


int main (void) {
   char buf[40];

//set up uart as stdio

   uart=fdevopen(uart_putchar, uart_getchar);

   init_uart();

// _delay_ms(1);     //may be needed with some PCs for synch

   puts("Hello, world!");

   while (1)
   {
   scanf("%s",buf);  //input some characters (terminated by white space)
   printf("string input: %s\n", buf);
   }
   return 0;
}
