/*
Example combining RS232 character IO and use of stdio printf routines
For the Orangutan with the ATmega168 processor.
Note that the UART register and bit names are different than for an Orangutan
containing the ATmega8 -- there is more than one uart on later processors, 
(see below)
*/

#include <stdio.h>
#include <avr/io.h>

// set correct CPU clock speed and desired baud rate

#define F_CPU 8000000UL
#define BAUDRATE 9600
 
#include <util/delay.h>

// set up stdio to be the UART

int uart_putchar(char c, FILE *stream);
int uart_getchar(FILE *stream);
FILE uart_stdio = FDEV_SETUP_STREAM(uart_putchar, uart_getchar, _FDEV_SETUP_RW);

//initialize the UART hardware on an ATmega168. 
//For the ATmega8, take the "0" out of the bit and register names.
//(refer to data sheet for naming conventions)

void init_uart(void)
{
   UCSR0B = (1<<RXEN0) | (1<<TXEN0);  //enable transmitter and receiver
   UCSR0C = (3<<UCSZ00); // 8 bit, 1 stop bit, no parity
   UBRR0H = ((F_CPU / (16UL * BAUDRATE) - 1)>>8);  //0 for 8 MHz
   UBRR0L =  (F_CPU / (16UL * BAUDRATE) - 1);  //51 for 8 MHz
}

int uart_putchar(char c, FILE *stream)
{
   loop_until_bit_is_set(UCSR0A, UDRE0); // wait for transmit register to clear
   UDR0 = c;    //send the character
   return 0;
}
int uart_getchar(FILE *stream)
{
	volatile unsigned char c;

	loop_until_bit_is_set(UCSR0A, RXC0);  //wait until receive complete

// optional error check in following
   if ((UCSR0A & (_BV(FE0) | _BV(DOR0))) != 0 ) return _FDEV_ERR;  //frame or data overrun error

   c = UDR0;
   return c;
}

int main(void)
{
	char c;

	init_uart();    // Initialize the uart
	stdout = stdin = &uart_stdio;    //connect stdio to the uart

	_delay_ms(1); //some PCs need a delay here for synchronization

	printf("Hello, world!\n");
	
	do 
	{
	c=uart_getchar(NULL);
	if(c >= ' ') uart_putchar(c,NULL); //echo if printable
	}
	while (c != 0x1B);  //exit loop if escape character received
	return 0;
}
